@@ -17,7 +17,9 @@
 #include <minizinc/iter.hh>
 #include <minizinc/model.hh>
 #include <minizinc/prettyprinter.hh>
+#include <minizinc/values.hh>
 
+#include <algorithm>
 #include <limits>
 
 namespace MiniZinc {
@@ -952,6 +954,93 @@ void Call::args(const std::vector<Expression*>& args) {
   }
 }
 
+/// Constructor to create commutative sorted call
+Call* Call::commutativeNormalized(EnvI& env, const Call* orig) {
+  auto com_sort = [](Expression* x, Expression* y) {
+    if (Expression::eid(x) != Expression::eid(y)) {
+      return Expression::eid(x) < Expression::eid(y);
+    }
+    switch (Expression::eid(x)) {
+      case Expression::E_ID: {
+        return Expression::cast<Id>(x)->str() < Expression::cast<Id>(y)->str();
+      }
+      case Expression::E_BOOLLIT: {
+        return static_cast<int>(Expression::cast<BoolLit>(x)->v()) <
+               static_cast<int>(Expression::cast<BoolLit>(y)->v());
+      }
+      case Expression::E_INTLIT: {
+        return IntLit::v(Expression::cast<IntLit>(x)) < IntLit::v(Expression::cast<IntLit>(y));
+      }
+      case Expression::E_FLOATLIT: {
+        return FloatLit::v(Expression::cast<FloatLit>(x)) <
+               FloatLit::v(Expression::cast<FloatLit>(y));
+      }
+      case Expression::E_STRINGLIT: {
+        return Expression::cast<StringLit>(x)->v() < Expression::cast<StringLit>(y)->v();
+      }
+      case Expression::E_SETLIT: {
+        if (Expression::type(x).bt() == Type::BT_INT) {
+          IntSetVal* xs = Expression::cast<SetLit>(x)->isv();
+          IntSetVal* ys = Expression::cast<SetLit>(y)->isv();
+          if (xs->size() != ys->size()) {
+            return xs->size() < ys->size();
+          }
+          for (unsigned int i = 0; i < xs->size(); ++i) {
+            if (xs->min(i) != ys->min(i)) {
+              return xs->min(i) < ys->min(i);
+            }
+            if (xs->max(i) != ys->max(i)) {
+              return xs->max(i) < ys->max(i);
+            }
+          }
+          return true;  // equal
+        }
+        FloatSetVal* xs = Expression::cast<SetLit>(x)->fsv();
+        FloatSetVal* ys = Expression::cast<SetLit>(y)->fsv();
+        if (xs->size() != ys->size()) {
+          return xs->size() < ys->size();
+        }
+        for (unsigned int i = 0; i < xs->size(); ++i) {
+          if (xs->min(i) != ys->min(i)) {
+            return xs->min(i) < ys->min(i);
+          }
+          if (xs->max(i) != ys->max(i)) {
+            return xs->max(i) < ys->max(i);
+          }
+        }
+        return true;  // equal
+      }
+      default: {
+        return x < y;
+      }
+    }
+  };
+
+  assert(orig->argCount() > 0);
+  Call* c = nullptr;
+  if (orig->argCount() == 1) {
+    assert(Expression::type(orig->arg(0)).dim() != 0);
+    ArrayLit* al = eval_array_lit(env, orig->arg(0));
+    std::vector<Expression*> elem(al->size());
+    for (unsigned int i = 0; i < al->size(); ++i) {
+      elem[i] = (*al)[i];
+    }
+    std::sort(elem.begin(), elem.end(), com_sort);
+    auto* arg = new ArrayLit(Expression::loc(al), elem);
+    arg->type(Expression::type(al));
+    c = Call::a(Expression::loc(orig), orig->id(), {arg});
+  } else {
+    std::vector<Expression*> args(orig->argCount());
+    for (unsigned int i = 0; i < orig->argCount(); ++i) {
+      args[i] = orig->arg(i);
+    }
+    std::sort(args.begin(), args.end(), com_sort);
+    c = Call::a(Expression::loc(orig), orig->id(), args);
+  }
+  c->decl(orig->decl());
+  return c;
+}
+
 void VarDecl::trail() {
   GC::trail(&_e, e());
   if (!_ti->ranges().empty()) {
@@ -1133,6 +1222,27 @@ void TypeInst::mkVar(const EnvI& env) {
   tt.ti(Type::TI_VAR);
   type(tt);
 }
+
+void TypeInst::mkPar(EnvI& env) {
+  Type tt = type();
+  tt.mkPar(env);
+  std::vector<std::pair<TypeInst*, Type>> todo({{this, tt}});
+  while (!todo.empty()) {
+    auto it = todo.back();
+    todo.pop_back();
+    it.first->type(it.second);
+    if (it.second.structBT()) {
+      auto* al = Expression::cast<ArrayLit>(it.first->domain());
+      al->type(it.second);
+      auto* st = env.getStructType(it.second);
+      assert(st->size() == al->size());
+      for (unsigned int i = 0; i < al->size(); i++) {
+        todo.emplace_back(Expression::cast<TypeInst>((*al)[i]), (*st)[i]);
+      }
+    }
+  }
+}
+
 void TypeInst::setStructDomain(const EnvI& env, const Type& struct_type, bool setTypeAny,
                                bool setTIRanges) {
   GCLock lock;
@@ -1326,8 +1436,8 @@ Type get_type(const Type& t) { return t; }
 const Location& get_loc(Expression* e, Expression* call, FunctionI* /*fi*/) {
   return Expression::loc(e).isNonAlloc() ? Expression::loc(call) : Expression::loc(e);
 }
-const Location& get_loc(const Type& /*t*/, Expression* /*call*/, FunctionI* fi) {
-  return fi->loc();
+const Location& get_loc(const Type& /*t*/, Expression* call, FunctionI* fi) {
+  return Expression::loc(call).isNonAlloc() ? fi->loc() : Expression::loc(call);
 }
 
 bool isa_tiid(Expression* e) {
@@ -1395,13 +1505,16 @@ Type return_type(EnvI& env, FunctionI* fi, const std::vector<T>& ta, Expression*
           }
           if (tiit_par.bt() == Type::BT_TOP || tiit_par.bt() == Type::BT_BOT) {
             tiit_par.bt(its_par.bt());
+            tiit_par.typeId(its_par.typeId());
           }
           if (its_par.bt() == Type::BT_TOP || its_par.bt() == Type::BT_BOT) {
             its_par.bt(tiit_par.bt());
+            its_par.typeId(tiit_par.typeId());
           }
           if (env.isSubtype(tiit_par, its_par, strictEnum)) {
             if (it->second.first.bt() == Type::BT_TOP) {
               it->second.first.bt(tiit.bt());
+              it->second.first.typeId(tiit.typeId());
             }
           } else if (env.isSubtype(its_par, tiit_par, strictEnum)) {
             it->second.first.bt(tiit_par.bt());
@@ -1498,6 +1611,7 @@ Type type_from_tmap(EnvI& env, TypeInst* ti, const ASTStringMap<std::pair<Type,
     std::vector<Type> fields(al->size());
     for (unsigned int i = 0; i < al->size(); i++) {
       fields[i] = type_from_tmap(env, Expression::cast<TypeInst>((*al)[i]), tmap);
+      ret.cv(ret.cv() || fields[i].cv());
     }
     unsigned int typeId = 0;
     if (ret.bt() == Type::BT_TUPLE) {
@@ -1555,6 +1669,7 @@ Type type_from_tmap(EnvI& env, TypeInst* ti, const ASTStringMap<std::pair<Type,
       ret.typeId(env.registerArrayEnum(enumIds));
     } else {
       ret.typeId(it->second.first.typeId());
+      ret.cv(ret.cv() || it->second.first.cv());
     }
   }
   if (!rh.empty()) {
@@ -1574,7 +1689,7 @@ Type type_from_tmap(EnvI& env, TypeInst* ti, const ASTStringMap<std::pair<Type,
       for (unsigned int i = 0; i < enumIds.size() - 1; i++) {
         enumIds[i] = orig_enumIds[i];
       }
-      if (curTypeId != 0 && ret.dim() > 0) {
+      if (curTypeId != 0 && ret.dim() != 0) {
         const auto& curIds = env.getArrayEnum(curTypeId);
         curTypeId = curIds[curIds.size() - 1];
       }
@@ -1608,6 +1723,9 @@ Type type_from_tmap(EnvI& env, TypeInst* ti, const ASTStringMap<std::pair<Type,
       ret.typeId(env.registerArrayEnum(enumIds));
     }
   }
+  if (ti->type().isPar()) {
+    ret.mkPar(env);
+  }
   return ret;
 }
 
@@ -2007,6 +2125,7 @@ Constants::Constants() {
   ids.bool2int = addString("bool2int");
   ids.int2float = addString("int2float");
   ids.bool2float = addString("bool2float");
+  ids.set2iter = addString("set2iter");
   ids.assert = addString("assert");
   ids.assert_dbg = addString("assert_dbg");
 
@@ -2028,10 +2147,11 @@ Constants::Constants() {
   ids.mzn_in_root_context = addString("mzn_in_root_context");
   ids.mzn_output_section = addString("mzn_output_section");
   ids.output_to_section = addString("output_to_section");
+  ids.output_to_json_section = addString("output_to_json_section");
   ids.mzn_default = addString("default");
   ids.trace = addString("trace");
   ids.trace_dbg = addString("trace_dbg");
-  ids.trace_to_section = addString("trace_to_section");
+  ids.mzn_trace_to_section = addString("mzn_trace_to_section");
 
   ids.array1d = addString("array1d");
   ids.array2d = addString("array2d");
@@ -2239,6 +2359,15 @@ Constants::Constants() {
   ann.flatzinc_builtin->type(Type::ann());
   ann.mzn_evaluate_once = addId("mzn_evaluate_once");
   ann.mzn_evaluate_once->type(Type::ann());
+  ann.promise_commutative = addId("promise_commutative");
+  ann.promise_commutative->type(Type::ann());
+  ann.seq_search = addString("seq_search");
+  ann.int_search = addString("int_search");
+  ann.bool_search = addString("bool_search");
+  ann.float_search = addString("float_search");
+  ann.set_search = addString("set_search");
+  ann.warm_start = addString("warm_start");
+  ann.warm_start_array = addString("warm_start_array");
 
   cli.cmdlineData_short_str = addString("-D");
   cli.cmdlineData_str = addString("--cmdline-data");
@@ -2323,7 +2452,8 @@ bool Constants::isCallByReferenceId(const ASTString& cid) const {
   return (cid == ids.assert || cid == ids.assert_dbg || cid == ids.trace || cid == ids.trace_dbg ||
           cid == "trace_exp" || cid == ids.mzn_symmetry_breaking_constraint ||
           cid == ids.mzn_redundant_constraint || cid == ids.mzn_default ||
-          cid == ids.mzn_deprecate || cid == ids.output_to_section || cid == ids.output);
+          cid == ids.mzn_deprecate || cid == ids.output_to_section ||
+          cid == ids.output_to_json_section || cid == ids.output);
 }
 
 void Constants::mark() {
@@ -2371,6 +2501,8 @@ bool Annotation::contains(Expression* e) const { return (_s != nullptr) && _s->c
 
 bool Annotation::isEmpty() const { return _s == nullptr || _s->isEmpty(); }
 
+unsigned int Annotation::size() const { return _s == nullptr ? 0 : _s->size(); }
+
 ExpressionSetIter Annotation::begin() const {
   return _s == nullptr ? ExpressionSetIter(true) : _s->begin();
 }
