@@ -450,7 +450,7 @@ public:
               } else {
                 _os << "(≡⊥)";
               }
-            } catch (EvalError) {
+            } catch (EvalError) { /* NOLINT(bugprone-empty-catch) */
               // nothing to print
             }
           }
@@ -460,7 +460,14 @@ public:
         const auto* fa = Expression::cast<FieldAccess>(e);
         p(fa->v());
         _os << ".";
-        p(fa->field());
+        if (Expression::isa<IntLit>(fa->field()) && Expression::type(fa->v()).isrecord()) {
+          // Has been turned into field number, so need to convert back into name
+          auto* rt = _env->getRecordType(Expression::type(fa->v()));
+          auto* i = Expression::cast<IntLit>(fa->field());
+          _os << Printer::quoteId(rt->fieldName(IntLit::v(i).toInt() - 1));
+        } else {
+          p(fa->field());
+        }
       } break;
       case Expression::E_COMP: {
         const auto* c = Expression::cast<Comprehension>(e);
@@ -2497,8 +2504,9 @@ void FznJSONPrinter::print(MiniZinc::Model* m) {
     printBasicElement(_os, m->solveItem()->e());
   }
   printAnnotations(_os, m->solveItem()->ann());
-  _os << " }\n";
-  _os << "}\n";
+  _os << " },\n"
+      << "  \"version\": \"1.0\"\n"
+      << "}\n";
 }
 
 }  // namespace MiniZinc
